@@ -1,6 +1,6 @@
 /*
  * sandbox.c: Process sandboxing
- *  
+ *
  * Copyright (C) 2017 Colin Watson.
  *
  * This file is part of man-db.
@@ -63,10 +63,14 @@
 #  include <seccomp.h>
 #endif /* HAVE_LIBSECCOMP */
 
+#include "attribute.h"
+#include "xalloc.h"
+#include "xstrndup.h"
+
 #include "manconfig.h"
 
-#include "error.h"
-
+#include "debug.h"
+#include "fatal.h"
 #include "sandbox.h"
 
 struct man_sandbox {
@@ -190,7 +194,13 @@ static bool can_load_seccomp (void)
 		if (nr == __NR_SCMP_ERROR) \
 			break; \
 		if (seccomp_rule_add (ctx, SCMP_ACT_ALLOW, nr, 0) < 0) \
-			error (FATAL, errno, "can't add seccomp rule"); \
+			fatal (errno, "can't add seccomp rule"); \
+	} while (0)
+
+#define SC_ALLOW_PERMISSIVE(name) \
+	do { \
+		if (permissive) \
+			SC_ALLOW (name); \
 	} while (0)
 
 #define SC_ALLOW_ARG_1(name, cmp1) \
@@ -199,7 +209,7 @@ static bool can_load_seccomp (void)
 		if (nr == __NR_SCMP_ERROR) \
 			break; \
 		if (seccomp_rule_add (ctx, SCMP_ACT_ALLOW, nr, 1, cmp1) < 0) \
-			error (FATAL, errno, "can't add seccomp rule"); \
+			fatal (errno, "can't add seccomp rule"); \
 	} while (0)
 
 #define SC_ALLOW_ARG_2(name, cmp1, cmp2) \
@@ -209,7 +219,7 @@ static bool can_load_seccomp (void)
 			break; \
 		if (seccomp_rule_add (ctx, SCMP_ACT_ALLOW, nr, \
 				      2, cmp1, cmp2) < 0) \
-			error (FATAL, errno, "can't add seccomp rule"); \
+			fatal (errno, "can't add seccomp rule"); \
 	} while (0)
 
 /* Create a seccomp filter.
@@ -231,10 +241,13 @@ static scmp_filter_ctx make_seccomp_filter (int permissive)
 #endif /* O_TMPFILE */
 		;
 
+	if (!can_load_seccomp ())
+		return NULL;
+
 	debug ("initialising seccomp filter (permissive: %d)\n", permissive);
 	ctx = seccomp_init (SCMP_ACT_ERRNO (ENOSYS));
 	if (!ctx)
-		error (FATAL, errno, "can't initialise seccomp filter");
+		fatal (errno, "can't initialise seccomp filter");
 
 	/* Allow sibling architectures for x86, since people sometimes mix
 	 * and match architectures there for performance reasons.
@@ -362,16 +375,16 @@ static scmp_filter_ctx make_seccomp_filter (int permissive)
 	SC_ALLOW ("fstatfs64");
 	SC_ALLOW ("ftruncate");
 	SC_ALLOW ("ftruncate64");
-	if (permissive) SC_ALLOW ("futimesat");
+	SC_ALLOW_PERMISSIVE ("futimesat");
 	SC_ALLOW ("getcwd");
 	SC_ALLOW ("getdents");
 	SC_ALLOW ("getdents64");
-	if (permissive) SC_ALLOW ("link");
-	if (permissive) SC_ALLOW ("linkat");
+	SC_ALLOW_PERMISSIVE ("link");
+	SC_ALLOW_PERMISSIVE ("linkat");
 	SC_ALLOW ("lstat");
 	SC_ALLOW ("lstat64");
-	if (permissive) SC_ALLOW ("mkdir");
-	if (permissive) SC_ALLOW ("mkdirat");
+	SC_ALLOW_PERMISSIVE ("mkdir");
+	SC_ALLOW_PERMISSIVE ("mkdirat");
 	SC_ALLOW ("mmap");
 	SC_ALLOW ("mmap2");
 	SC_ALLOW ("munmap");
@@ -410,24 +423,24 @@ static scmp_filter_ctx make_seccomp_filter (int permissive)
 	}
 	SC_ALLOW ("readlink");
 	SC_ALLOW ("readlinkat");
-	if (permissive) SC_ALLOW ("rename");
-	if (permissive) SC_ALLOW ("renameat");
-	if (permissive) SC_ALLOW ("renameat2");
-	if (permissive) SC_ALLOW ("rmdir");
+	SC_ALLOW_PERMISSIVE ("rename");
+	SC_ALLOW_PERMISSIVE ("renameat");
+	SC_ALLOW_PERMISSIVE ("renameat2");
+	SC_ALLOW_PERMISSIVE ("rmdir");
 	SC_ALLOW ("stat");
 	SC_ALLOW ("stat64");
 	SC_ALLOW ("statfs");
 	SC_ALLOW ("statfs64");
 	SC_ALLOW ("statx");
-	if (permissive) SC_ALLOW ("symlink");
-	if (permissive) SC_ALLOW ("symlinkat");
-	if (permissive) SC_ALLOW ("truncate");
-	if (permissive) SC_ALLOW ("truncateat");
-	if (permissive) SC_ALLOW ("unlink");
-	if (permissive) SC_ALLOW ("unlinkat");
-	if (permissive) SC_ALLOW ("utime");
-	if (permissive) SC_ALLOW ("utimensat");
-	if (permissive) SC_ALLOW ("utimes");
+	SC_ALLOW_PERMISSIVE ("symlink");
+	SC_ALLOW_PERMISSIVE ("symlinkat");
+	SC_ALLOW_PERMISSIVE ("truncate");
+	SC_ALLOW_PERMISSIVE ("truncateat");
+	SC_ALLOW_PERMISSIVE ("unlink");
+	SC_ALLOW_PERMISSIVE ("unlinkat");
+	SC_ALLOW_PERMISSIVE ("utime");
+	SC_ALLOW_PERMISSIVE ("utimensat");
+	SC_ALLOW_PERMISSIVE ("utimes");
 
 	/* systemd: SystemCallFilter=@io-event */
 	SC_ALLOW ("_newselect");
@@ -584,12 +597,14 @@ static void _sandbox_load (man_sandbox *sandbox, int permissive) {
 	if (can_load_seccomp ()) {
 		scmp_filter_ctx ctx;
 
-		debug ("loading seccomp filter (permissive: %d)\n",
-		       permissive);
 		if (permissive)
 			ctx = sandbox->permissive_ctx;
 		else
 			ctx = sandbox->ctx;
+		if (!ctx)
+			return;
+		debug ("loading seccomp filter (permissive: %d)\n",
+		       permissive);
 		if (seccomp_load (ctx) < 0) {
 			if (errno == EINVAL || errno == EFAULT) {
 				/* The kernel doesn't give us particularly
@@ -606,14 +621,13 @@ static void _sandbox_load (man_sandbox *sandbox, int permissive) {
 				/* Don't try this again. */
 				seccomp_filter_unavailable = 1;
 			} else
-				error (FATAL, errno,
-				       "can't load seccomp filter");
+				fatal (errno, "can't load seccomp filter");
 		}
 	}
 }
 #else /* !HAVE_LIBSECCOMP */
-static void _sandbox_load (man_sandbox *sandbox _GL_UNUSED,
-			   int permissive _GL_UNUSED)
+static void _sandbox_load (man_sandbox *sandbox MAYBE_UNUSED,
+			   int permissive MAYBE_UNUSED)
 {
 }
 #endif /* HAVE_LIBSECCOMP */
@@ -641,8 +655,10 @@ void sandbox_free (void *data) {
 	man_sandbox *sandbox = data;
 
 #ifdef HAVE_LIBSECCOMP
-	seccomp_release (sandbox->ctx);
-	seccomp_release (sandbox->permissive_ctx);
+	if (sandbox->ctx)
+		seccomp_release (sandbox->ctx);
+	if (sandbox->permissive_ctx)
+		seccomp_release (sandbox->permissive_ctx);
 #endif /* HAVE_LIBSECCOMP */
 
 	free (sandbox);
