@@ -25,17 +25,21 @@
 #  include "config.h"
 #endif /* HAVE_CONFIG_H */
 
+#include <assert.h>
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
 #include <unistd.h>
 
+#include "attribute.h"
+#include "error.h"
 #include "timespec.h"
+#include "xalloc.h"
 #include "xvasprintf.h"
 
 #include "manconfig.h"
 
-#include "error.h"
+#include "debug.h"
 
 #include "mydbm.h"
 #include "db_storage.h"
@@ -49,7 +53,7 @@
  * If promote_links is true, consider SO_MAN equivalent to ULT_MAN. This is
  * appropriate when sorting candidate pages for display.
  */
-int _GL_ATTRIBUTE_CONST compare_ids (char a, char b, int promote_links)
+int ATTRIBUTE_CONST compare_ids (char a, char b, int promote_links)
 {
 #ifdef FAVOUR_STRAYCATS
 	if (a == WHATIS_MAN && b == STRAY_CAT)
@@ -122,6 +126,7 @@ static datum make_content (struct mandata *in)
 {
 	datum cont;
 	static const char dash[] = "-";
+	char *value;
 
 	memset (&cont, 0, sizeof cont);
 
@@ -134,7 +139,7 @@ static datum make_content (struct mandata *in)
 	if (!in->whatis)
 		in->whatis = dash + 1;
 
-	MYDBM_SET (cont, xasprintf (
+	value = xasprintf (
 		"%s\t%s\t%s\t%ld\t%ld\t%c\t%s\t%s\t%s\t%s",
 		dash_if_unset (in->name),
 		in->ext,
@@ -145,7 +150,9 @@ static datum make_content (struct mandata *in)
 		in->pointer,
 		in->filter,
 		in->comp,
-		in->whatis));
+		in->whatis);
+	assert (value);
+	MYDBM_SET (cont, value);
 
 #ifdef NDBM
 	/* limit of 4096 bytes of data using ndbm */
@@ -190,6 +197,7 @@ static datum make_content (struct mandata *in)
 int dbstore (MYDBM_FILE dbf, struct mandata *in, const char *base)
 {
 	datum oldkey, oldcont;
+	char *value;
 
 	memset (&oldkey, 0, sizeof oldkey);
 	memset (&oldcont, 0, sizeof oldcont);
@@ -260,8 +268,10 @@ int dbstore (MYDBM_FILE dbf, struct mandata *in, const char *base)
 		MYDBM_FREE_DPTR (newkey);
 		MYDBM_FREE_DPTR (newcont);
 
-		MYDBM_SET (newcont, xasprintf (
-			"%s\t%s\t%s", MYDBM_DPTR (oldcont), base, in->ext));
+		value = xasprintf (
+			"%s\t%s\t%s", MYDBM_DPTR (oldcont), base, in->ext);
+		assert (value);
+		MYDBM_SET (newcont, value);
 		MYDBM_FREE_DPTR (oldcont);
 
 		/* Try to replace the old simple data with the new stuff */
@@ -349,8 +359,10 @@ int dbstore (MYDBM_FILE dbf, struct mandata *in, const char *base)
 
 		/* Now build a simple reference to the above two items */
 
-		MYDBM_SET (newcont, xasprintf (
-			"\t%s\t%s\t%s\t%s", old_name, old.ext, base, in->ext));
+		value = xasprintf (
+			"\t%s\t%s\t%s\t%s", old_name, old.ext, base, in->ext);
+		assert (value);
+		MYDBM_SET (newcont, value);
 
 		if (MYDBM_REPLACE (dbf, oldkey, newcont))
 			gripe_replace_key (dbf, MYDBM_DPTR (oldkey));
