@@ -23,7 +23,7 @@
  *
  * Currently satisfies:
  *
- *	*hash based* 
+ *	*hash based*
  *		GNU dbm: 	(gdbm & ndbm)
  *		Berkeley db: 	(ndbm)
  *		`native': 	(ndbm)
@@ -31,13 +31,16 @@
  *	*binary tree based*
  *		Berkeley db: 	(BTREE)
  *
- * Tue Apr 26 12:56:44 BST 1994  Wilf. (G.Wilford@ee.surrey.ac.uk) 
+ * Tue Apr 26 12:56:44 BST 1994  Wilf. (G.Wilford@ee.surrey.ac.uk)
  */
 
 #ifndef MYDBM_H
 # define MYDBM_H
 
+# include <stdbool.h>
+
 # include "timespec.h"
+# include "xvasprintf.h"
 
 # if defined(GDBM) && !defined(NDBM) && !defined(BTREE)
 
@@ -53,39 +56,38 @@ extern int gdbm_exists (GDBM_FILE db, datum key);
 typedef struct {
 	char *name;
 	GDBM_FILE file;
+	struct timespec *mtime;
 } *man_gdbm_wrapper;
 
-man_gdbm_wrapper man_gdbm_open_wrapper (const char *name, int flags);
+man_gdbm_wrapper man_gdbm_new (const char *name);
+bool man_gdbm_open_wrapper (man_gdbm_wrapper wrap, int flags);
 datum man_gdbm_firstkey (man_gdbm_wrapper wrap);
 datum man_gdbm_nextkey (man_gdbm_wrapper wrap, datum key);
 struct timespec man_gdbm_get_time (man_gdbm_wrapper wrap);
-void man_gdbm_set_time (man_gdbm_wrapper wrap, const struct timespec time);
-void man_gdbm_close (man_gdbm_wrapper wrap);
+void man_gdbm_free (man_gdbm_wrapper wrap);
 
 #  define BLK_SIZE			0  /* to invoke normal fs block size */
 #  define DB_EXT				".db"
 #  define MYDBM_FILE 			man_gdbm_wrapper
+#  define MYDBM_NEW(file)		man_gdbm_new(file)
 #  define MYDBM_DPTR(d)			((d).dptr)
 #  define MYDBM_SET_DPTR(d, value)	((d).dptr = (value))
 #  define MYDBM_DSIZE(d)		((d).dsize)
-#  define MYDBM_CTRWOPEN(file)		\
-	man_gdbm_open_wrapper(file, GDBM_NEWDB|GDBM_FAST)
-#  define MYDBM_CRWOPEN(file)		\
-	man_gdbm_open_wrapper(file, GDBM_WRCREAT|GDBM_FAST)
-#  define MYDBM_RWOPEN(file)		\
-	man_gdbm_open_wrapper(file, GDBM_WRITER|GDBM_FAST)
-#  define MYDBM_RDOPEN(file)		\
-	man_gdbm_open_wrapper(file, GDBM_READER)
+#  define MYDBM_CTRWOPEN(wrap)		\
+	man_gdbm_open_wrapper(wrap, GDBM_NEWDB|GDBM_FAST)
+#  define MYDBM_RWOPEN(wrap)		\
+	man_gdbm_open_wrapper(wrap, GDBM_WRITER|GDBM_FAST)
+#  define MYDBM_RDOPEN(wrap)		\
+	man_gdbm_open_wrapper(wrap, GDBM_READER)
 #  define MYDBM_INSERT(db, key, cont)	gdbm_store((db)->file, key, cont, GDBM_INSERT)
 #  define MYDBM_REPLACE(db, key, cont) 	gdbm_store((db)->file, key, cont, GDBM_REPLACE)
 #  define MYDBM_EXISTS(db, key)		gdbm_exists((db)->file, key)
 #  define MYDBM_DELETE(db, key)		gdbm_delete((db)->file, key)
 #  define MYDBM_FETCH(db, key)		gdbm_fetch((db)->file, key)
-#  define MYDBM_CLOSE(db)		man_gdbm_close(db)
+#  define MYDBM_FREE(db)		man_gdbm_free(db)
 #  define MYDBM_FIRSTKEY(db)		man_gdbm_firstkey(db)
 #  define MYDBM_NEXTKEY(db, key)		man_gdbm_nextkey(db, key)
 #  define MYDBM_GET_TIME(db)		man_gdbm_get_time(db)
-#  define MYDBM_SET_TIME(db, time)	man_gdbm_set_time(db, time)
 #  define MYDBM_REORG(db)		gdbm_reorganize((db)->file)
 
 # elif defined(NDBM) && !defined(GDBM) && !defined(BTREE)
@@ -101,35 +103,35 @@ void man_gdbm_close (man_gdbm_wrapper wrap);
 typedef struct {
 	char *name;
 	DBM *file;
+	struct timespec *mtime;
 } *man_ndbm_wrapper;
 
-extern man_ndbm_wrapper man_ndbm_open (const char *name, int flags, int mode);
+extern man_ndbm_wrapper man_ndbm_new (const char *name);
+extern bool man_ndbm_open (man_ndbm_wrapper wrap, int flags, int mode);
 extern datum man_ndbm_firstkey (man_ndbm_wrapper wrap);
 extern datum man_ndbm_nextkey (man_ndbm_wrapper wrap, datum key);
 extern struct timespec man_ndbm_get_time (man_ndbm_wrapper wrap);
-extern void man_ndbm_set_time (man_ndbm_wrapper wrap, const struct timespec time);
-extern void man_ndbm_close (man_ndbm_wrapper wrap);
+extern void man_ndbm_free (man_ndbm_wrapper wrap);
 
 #  define DB_EXT				""
 #  define MYDBM_FILE 			man_ndbm_wrapper
+#  define MYDBM_NEW(file)		man_ndbm_new(file)
 #  define MYDBM_DPTR(d)			((d).dptr)
 #  define MYDBM_SET_DPTR(d, value)	((d).dptr = (value))
 #  define MYDBM_DSIZE(d)		((d).dsize)
-#  define MYDBM_CTRWOPEN(file)		man_ndbm_open(file, O_TRUNC|O_CREAT|O_RDWR, DBMODE)
-#  define MYDBM_CRWOPEN(file)		man_ndbm_open(file, O_CREAT|O_RDWR, DBMODE)
-#  define MYDBM_RWOPEN(file)		man_ndbm_open(file, O_RDWR, DBMODE)
-#  define MYDBM_RDOPEN(file)		man_ndbm_open(file, O_RDONLY, DBMODE)
+#  define MYDBM_CTRWOPEN(wrap)		man_ndbm_open(wrap, O_TRUNC|O_CREAT|O_RDWR, DBMODE)
+#  define MYDBM_RWOPEN(wrap)		man_ndbm_open(wrap, O_RDWR, DBMODE)
+#  define MYDBM_RDOPEN(wrap)		man_ndbm_open(wrap, O_RDONLY, DBMODE)
 #  define MYDBM_INSERT(db, key, cont)	dbm_store((db)->file, key, cont, DBM_INSERT)
 #  define MYDBM_REPLACE(db, key, cont)	dbm_store((db)->file, key, cont, DBM_REPLACE)
 #  define MYDBM_EXISTS(db, key)		(dbm_fetch((db)->file, key).dptr != NULL)
 #  define MYDBM_DELETE(db, key)		dbm_delete((db)->file, key)
 #  define MYDBM_FETCH(db, key)		copy_datum(dbm_fetch((db)->file, key))
-#  define MYDBM_CLOSE(db)		man_ndbm_close(db)
+#  define MYDBM_FREE(db)		man_ndbm_free(db)
 #  define MYDBM_FIRSTKEY(db)		man_ndbm_firstkey(db)
 #  define MYDBM_NEXTKEY(db, key)	man_ndbm_nextkey(db, key)
 #  define MYDBM_GET_TIME(db)		man_ndbm_get_time(db)
-#  define MYDBM_SET_TIME(db, time)	man_ndbm_set_time(db, time)
-#  define MYDBM_REORG(db)		/* nothing - not implemented */
+#  define MYDBM_REORG(db)		do {} while (0) /* not implemented */
 
 # elif defined(BTREE) && !defined(NDBM) && !defined(GDBM)
 
@@ -141,13 +143,14 @@ extern void man_ndbm_close (man_ndbm_wrapper wrap);
 typedef struct {
 	char *name;
 	DB *file;
+	struct timespec *mtime;
 } *man_btree_wrapper;
 
 typedef DBT datum;
 
-extern man_btree_wrapper man_btree_open (const char *filename, int flags,
-					 int mode);
-extern void man_btree_close (man_btree_wrapper wrap);
+extern man_btree_wrapper man_btree_new (const char *filename);
+extern bool man_btree_open (man_btree_wrapper wrap, int flags, int mode);
+extern void man_btree_free (man_btree_wrapper wrap);
 extern int man_btree_exists (man_btree_wrapper wrap, datum key);
 extern datum man_btree_fetch (man_btree_wrapper wrap, datum key);
 extern int man_btree_insert (man_btree_wrapper wrap, datum key, datum cont);
@@ -158,29 +161,26 @@ extern int man_btree_replace (man_btree_wrapper wrap,
 extern int man_btree_nextkeydata (man_btree_wrapper wrap,
 				  datum *key, datum *cont);
 extern struct timespec man_btree_get_time (man_btree_wrapper wrap);
-extern void man_btree_set_time (man_btree_wrapper wrap,
-				const struct timespec time);
 
 #  define DB_EXT			".bt"
 #  define MYDBM_FILE			man_btree_wrapper
+#  define MYDBM_NEW(file)		man_btree_new(file)
 #  define MYDBM_DPTR(d)			((char *) (d).data)
 #  define MYDBM_SET_DPTR(d, value)	((d).data = (char *) (value))
 #  define MYDBM_DSIZE(d)		((d).size)
-#  define MYDBM_CTRWOPEN(file)		man_btree_open(file, O_TRUNC|O_CREAT|O_RDWR, DBMODE)
-#  define MYDBM_CRWOPEN(file)		man_btree_open(file, O_CREAT|O_RDWR, DBMODE)
-#  define MYDBM_RWOPEN(file)		man_btree_open(file, O_RDWR, DBMODE)
-#  define MYDBM_RDOPEN(file)		man_btree_open(file, O_RDONLY, DBMODE)
+#  define MYDBM_CTRWOPEN(wrap)		man_btree_open(wrap, O_TRUNC|O_CREAT|O_RDWR, DBMODE)
+#  define MYDBM_RWOPEN(wrap)		man_btree_open(wrap, O_RDWR, DBMODE)
+#  define MYDBM_RDOPEN(wrap)		man_btree_open(wrap, O_RDONLY, DBMODE)
 #  define MYDBM_INSERT(db, key, cont)	man_btree_insert(db, key, cont)
 #  define MYDBM_REPLACE(db, key, cont)	man_btree_replace(db, key, cont)
 #  define MYDBM_EXISTS(db, key)		man_btree_exists(db, key)
 #  define MYDBM_DELETE(db, key)		(((db)->file->del)((db)->file, &key, 0) ? -1 : 0)
 #  define MYDBM_FETCH(db, key)		man_btree_fetch(db, key)
-#  define MYDBM_CLOSE(db)		man_btree_close(db)
+#  define MYDBM_FREE(db)		man_btree_free(db)
 #  define MYDBM_FIRSTKEY(db)		man_btree_firstkey(db)
 #  define MYDBM_NEXTKEY(db, key)	man_btree_nextkey(db)
 #  define MYDBM_GET_TIME(db)		man_btree_get_time(db)
-#  define MYDBM_SET_TIME(db, time)	man_btree_set_time(db, time)
-#  define MYDBM_REORG(db)		/* nothing - not implemented */
+#  define MYDBM_REORG(db)		do {} while (0) /* not implemented */
 
 # else /* not GDBM or NDBM or BTREE */
 #  error Define either GDBM, NDBM or BTREE before including mydbm.h
@@ -197,4 +197,7 @@ extern datum copy_datum (datum dat);
 extern void dbver_wr(MYDBM_FILE dbfile);
 extern int dbver_rd(MYDBM_FILE dbfile);
 
+#define MAN_DB		"/index" DB_EXT
+#define mkdbname(path)	xasprintf ("%s%s", path, MAN_DB)
+
 #endif /* MYDBM_H */
