@@ -29,8 +29,14 @@
 
 #include "gl_array_list.h"
 #include "gl_xlist.h"
+#include "xalloc.h"
+#include "xstrndup.h"
 
 #include "manconfig.h"
+
+#include "debug.h"
+#include "util.h"
+
 #include "descriptions.h"
 
 /* Free a page description. */
