@@ -28,15 +28,22 @@
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
+#include <sys/stat.h>
+#include <sys/types.h>
+#include <time.h>
+#include <unistd.h>
 
 #include "gettext.h"
 #define _(String) gettext (String)
 
 #include "error.h"
 #include "gl_list.h"
+#include "stat-time.h"
+#include "xalloc.h"
 
 #include "manconfig.h"
 
+#include "debug.h"
 #include "glcontainers.h"
 
 #include "db_storage.h"
@@ -69,15 +76,15 @@ void store_descriptions (MYDBM_FILE dbf, gl_list_t descs, struct mandata *info,
 {
 	const struct page_description *desc;
 	char save_id = info->id;
+	struct timespec save_mtime = info->mtime;
 	const char *trace_name;
 
 	if (gl_list_size (descs) && trace) {
-		GL_LIST_FOREACH_START (trace, trace_name)
+		GL_LIST_FOREACH (trace, trace_name)
 			debug ("trace: '%s'\n", trace_name);
-		GL_LIST_FOREACH_END (trace);
 	}
 
-	GL_LIST_FOREACH_START (descs, desc) {
+	GL_LIST_FOREACH (descs, desc) {
 		/* Either it's the real thing or merely a reference. Get the
 		 * id and pointer right in either case.
 		 */
@@ -88,9 +95,10 @@ void store_descriptions (MYDBM_FILE dbf, gl_list_t descs, struct mandata *info,
 			info->id = save_id;
 			info->pointer = NULL;
 			info->whatis = desc->whatis;
+			info->mtime = save_mtime;
 			found_real_page = true;
 		} else if (trace) {
-			GL_LIST_FOREACH_START (trace, trace_name) {
+			GL_LIST_FOREACH (trace, trace_name) {
 				struct mandata trace_info;
 				char *buf;
 
@@ -98,6 +106,8 @@ void store_descriptions (MYDBM_FILE dbf, gl_list_t descs, struct mandata *info,
 						     &trace_info, "");
 				if (trace_info.name &&
 				    STREQ (trace_info.name, desc->name)) {
+					struct stat st;
+
 					if (path && !is_prefix (path, buf)) {
 						/* Link outside this manual
 						 * hierarchy; skip this
@@ -116,12 +126,17 @@ void store_descriptions (MYDBM_FILE dbf, gl_list_t descs, struct mandata *info,
 						info->id = save_id;
 					info->pointer = NULL;
 					info->whatis = desc->whatis;
+					if (lstat (trace_name, &st) == 0)
+						info->mtime = get_stat_mtime
+							(&st);
+					else
+						info->mtime = save_mtime;
 					found_real_page = true;
 				}
 
 				free (trace_info.name);
 				free (buf);
-			} GL_LIST_FOREACH_END (trace);
+			}
 		}
 
 		if (found_external) {
@@ -140,6 +155,7 @@ void store_descriptions (MYDBM_FILE dbf, gl_list_t descs, struct mandata *info,
 			 * more than once.
 			 */
 			info->whatis = NULL;
+			info->mtime = save_mtime;
 		}
 
 		debug ("name = '%s', id = %c\n", desc->name, info->id);
@@ -147,5 +163,5 @@ void store_descriptions (MYDBM_FILE dbf, gl_list_t descs, struct mandata *info,
 			gripe_bad_store (base, info->ext);
 			break;
 		}
-	} GL_LIST_FOREACH_END (descs);
+	}
 }
