@@ -1,6 +1,6 @@
 /*
  * globbing_test.c: test program for file-finding functions
- *  
+ *
  * Copyright (C) 1995 Graeme W. Wilford. (Wilf.)
  * Copyright (C) 2001, 2002, 2003, 2006, 2007, 2008 Colin Watson.
  *
@@ -25,11 +25,13 @@
 #  include "config.h"
 #endif /* HAVE_CONFIG_H */
 
+#include <assert.h>
 #include <stdbool.h>
 #include <stdio.h>
 #include <stdlib.h>
 
 #include "argp.h"
+#include "error.h"
 #include "gl_list.h"
 #include "progname.h"
 
@@ -38,8 +40,10 @@
 
 #include "manconfig.h"
 
-#include "error.h"
+#include "debug.h"
 #include "glcontainers.h"
+#include "util.h"
+
 #include "globbing.h"
 
 extern const char *extension;
@@ -95,6 +99,9 @@ static error_t parse_opt (int key, char *arg, struct argp_state *state)
 				argp_usage (state);
 			remaining_args = state->argv + state->next;
 			return 0;
+		case ARGP_KEY_NO_ARGS:
+			argp_usage (state);
+			break;
 	}
 	return ARGP_ERR_UNKNOWN;
 }
@@ -112,6 +119,7 @@ int main (int argc, char **argv)
 
 	if (argp_parse (&argp, argc, argv, 0, 0, 0))
 		exit (FAIL);
+	assert (remaining_args);
 
 	for (i = 0; i <= 1; i++) {
 		gl_list_t files;
@@ -122,9 +130,8 @@ int main (int argc, char **argv)
 				       (match_case ? LFF_MATCHCASE : 0) |
 				       (regex_opt ? LFF_REGEX : 0) |
 				       (wildcard ? LFF_WILDCARD : 0));
-		GL_LIST_FOREACH_START (files, file)
+		GL_LIST_FOREACH (files, file)
 			printf ("%s\n", file);
-		GL_LIST_FOREACH_END (files);
 		gl_list_free (files);
 	}
 	return 0;
