@@ -28,6 +28,7 @@
 #  include "config.h"
 #endif /* HAVE_CONFIG_H */
 
+#include <assert.h>
 #include <stdbool.h>
 #include <string.h>
 #include <stdlib.h>
@@ -41,15 +42,18 @@
 
 #include "canonicalize.h"
 #include "dirname.h"
+#include "error.h"
 #include "gl_array_list.h"
 #include "gl_xlist.h"
+#include "xalloc.h"
 
 #include "gettext.h"
 #define _(String) gettext (String)
 
 #include "manconfig.h"
 
-#include "error.h"
+#include "appendstr.h"
+#include "debug.h"
 #include "glcontainers.h"
 #include "pipeline.h"
 #include "decompress.h"
@@ -57,13 +61,17 @@
 #include "orderfiles.h"
 #include "sandbox.h"
 #include "security.h"
+#include "util.h"
 
 #include "mydbm.h"
 #include "db_storage.h"
 
+#include "compression.h"
 #include "descriptions.h"
+#include "lexgrog.h"
 #include "manp.h"
 #include "manconv_client.h"
+#include "straycats.h"
 #include "ult_src.h"
 
 extern man_sandbox *sandbox;
@@ -88,7 +96,7 @@ static int check_for_stray (MYDBM_FILE dbf)
 	names = new_string_list (GL_ARRAY_LIST, false);
 
 	while ((catlist = readdir (cdir)) != NULL) {
-		if (*catlist->d_name == '.' && 
+		if (*catlist->d_name == '.' &&
 		    strlen (catlist->d_name) < (size_t) 3)
 			continue;
 		gl_list_add_last (names, xstrdup (catlist->d_name));
@@ -102,7 +110,7 @@ static int check_for_stray (MYDBM_FILE dbf)
 	lenman = strlen (mandir);
 	lencat = strlen (catdir);
 
-	GL_LIST_FOREACH_START (names, name) {
+	GL_LIST_FOREACH (names, name) {
 		struct mandata info;
 		char *ext, *section;
 		short found;
@@ -135,7 +143,7 @@ static int check_for_stray (MYDBM_FILE dbf)
 		*(mandir + lenman - 1) = '/';
 
 		/* check for bogosity */
-		
+
 		if (!ext || strncmp (ext + 1, section, strlen (section)) != 0) {
 			if (quiet < 2)
 				error (0, 0,
@@ -152,16 +160,16 @@ static int check_for_stray (MYDBM_FILE dbf)
 
 		debug ("Testing for existence: %s\n", mandir);
 
-		if (stat (mandir, &buf) == 0) 
+		if (stat (mandir, &buf) == 0)
 			found = 1;
 		else if ((comp = comp_file (mandir))) {
 			found = 1;
 			free (comp->stem);
-		} else 
+		} else
 			found = 0;
 
 		if (!found) {
-			pipeline *decomp;
+			decompress *decomp;
 			struct mandata *exists;
 			lexgrog lg;
 			char *lang, *page_encoding;
@@ -177,7 +185,7 @@ static int check_for_stray (MYDBM_FILE dbf)
 			*(ext++) = '\0';
 			info.ext = ext;
 
-			/* see if we already have it, before going any 
+			/* see if we already have it, before going any
 			   further */
 			mandir_base = base_name (mandir);
 			exists = dblookup_exact (dbf, mandir_base, info.ext,
@@ -198,7 +206,7 @@ static int check_for_stray (MYDBM_FILE dbf)
 			info.mtime.tv_nsec = 0;
 
 			drop_effective_privs ();
-			decomp = decompress_open (catdir);
+			decomp = decompress_open (catdir, 0);
 			regain_effective_privs ();
 			if (!decomp) {
 				error (0, errno, _("can't open %s"), catdir);
@@ -208,12 +216,13 @@ static int check_for_stray (MYDBM_FILE dbf)
 			lang = lang_dir (mandir);
 			page_encoding = get_page_encoding (lang);
 			if (page_encoding)
-				add_manconv (decomp, page_encoding, "UTF-8");
+				add_manconv (decompress_get_pipeline (decomp),
+					     page_encoding, "UTF-8");
 			free (page_encoding);
 			free (lang);
 
 			col_cmd = pipecmd_new_argstr
-				(get_def_user ("col", COL));
+				(get_def_user ("col", PROG_COL));
 			pipecmd_arg (col_cmd, "-bx");
 			col_locale = find_charset_locale ("UTF-8");
 			if (col_locale) {
@@ -223,7 +232,8 @@ static int check_for_stray (MYDBM_FILE dbf)
 			}
 			pipecmd_pre_exec (col_cmd, sandbox_load, sandbox_free,
 					  sandbox);
-			pipeline_command (decomp, col_cmd);
+			pipeline_command (decompress_get_pipeline (decomp),
+					  col_cmd);
 
 			fullpath = canonicalize_file_name (catdir);
 			if (!fullpath) {
@@ -240,7 +250,7 @@ static int check_for_stray (MYDBM_FILE dbf)
 
 				free (fullpath);
 				drop_effective_privs ();
-				pipeline_start (decomp);
+				decompress_start (decomp);
 				regain_effective_privs ();
 
 				strays++;
@@ -265,14 +275,14 @@ static int check_for_stray (MYDBM_FILE dbf)
 			}
 
 			free (lg.whatis);
-			pipeline_free (decomp);
+			decompress_free (decomp);
 next_exists:
 			free_mandata_struct (exists);
 			free (mandir_base);
 		}
 next_section:
 		free (section);
-	} GL_LIST_FOREACH_END (names);
+	}
 	gl_list_free (names);
 	return strays;
 }
@@ -319,36 +329,26 @@ static int open_catdir (MYDBM_FILE dbf)
 	return strays;
 }
 
-int straycats (const char *database, const char *manpath)
+int straycats (MYDBM_FILE dbf, const char *manpath)
 {
-	MYDBM_FILE dbf;
 	char *catpath;
 	int strays;
 
-	dbf = MYDBM_RWOPEN (database);
-	if (dbf && dbver_rd (dbf)) {
-		MYDBM_CLOSE (dbf);
-		dbf = NULL;
-	}
-	if (!dbf) {
-		error (0, errno, _("warning: can't update index cache %s"),
-		       database);
-		return 0;
-	}
+	assert (dbf->file);
 
 	catpath = get_catpath (manpath, SYSTEM_CAT | USER_CAT);
 
 	/* look in the usual catpath location */
 	mandir = xstrdup (manpath);
 	catdir = xstrdup (manpath);
-	strays = open_catdir (dbf); 
+	strays = open_catdir (dbf);
 
-	/* look in the alternate catpath location if we have one 
+	/* look in the alternate catpath location if we have one
 	   and it's different from the usual catpath */
 
 	if (catpath)
 		debug ("catpath: %s, manpath: %s\n", catpath, manpath);
-		
+
 	if (catpath && strcmp (catpath, manpath) != 0) {
 		*mandir = *catdir = '\0';
 		mandir = appendstr (mandir, manpath, (void *) 0);
@@ -361,6 +361,5 @@ int straycats (const char *database, const char *manpath)
 
 	free (catpath);
 
-	MYDBM_CLOSE (dbf);
 	return strays;
 }
