@@ -47,6 +47,8 @@
 #include "dirname.h"
 #include "error.h"
 #include "gl_xlist.h"
+#include "xalloc.h"
+#include "xstrndup.h"
 #include "xvasprintf.h"
 
 #include "gettext.h"
@@ -54,9 +56,10 @@
 
 #include "manconfig.h"
 
-#include "pipeline.h"
-#include "decompress.h"
+#include "debug.h"
 
+#include "compression.h"
+#include "decompress.h"
 #include "globbing.h"
 #include "ult_src.h"
 
@@ -136,7 +139,7 @@ static char *ult_softlink (const char *fullpath)
 	return resolved_path;
 }
 
-/* Test 'buffer' to see if it contains a .so include. If so and it's not an 
+/* Test 'buffer' to see if it contains a .so include. If so and it's not an
  * absolute filename, return newly allocated string whose contents are the
  * include.
  */
@@ -153,19 +156,19 @@ static char *test_for_include (const char *buffer)
 	if (strncmp (buffer, ".so", 3) == 0) {
 		buffer += 3;
 
-		/* strip out any whitespace between the command and 
+		/* strip out any whitespace between the command and
 		   it's argumant */
 		while (CTYPE (isspace, *buffer))
 			buffer++;
 
 		/* If .so's argument is an absolute filename, it could be
 		 * either (i) a macro inclusion, (ii) a non local manual page
-		 * or (iii) a (somewhat bogus) reference to a local manual 
+		 * or (iii) a (somewhat bogus) reference to a local manual
 		 * page.
-		 * 
+		 *
 		 * If (i) or (ii), we must not follow the reference. (iii) is
-		 * a problem with the manual page, thus we don't want to 
-		 * follow any absolute inclusions in our quest for the 
+		 * a problem with the manual page, thus we don't want to
+		 * follow any absolute inclusions in our quest for the
 		 * ultimate source file */
 		if (*buffer != '/') {
 			const char *end = buffer;
@@ -188,6 +191,7 @@ static char *find_include (const char *name, const char *path,
 	 * case it went outside the mantree.
 	 */
 	ret = xasprintf ("%s/%s", path, include);
+	assert (ret);
 
 	/* If the original path from above doesn't exist, try to create new
 	 * path as if the "include" was relative to the current man page.
@@ -197,6 +201,7 @@ static char *find_include (const char *name, const char *path,
 
 	dirname = dir_name (name);
 	temp_file = xasprintf ("%s/%s", dirname, include);
+	assert (temp_file);
 	free (dirname);
 
 	if (CAN_ACCESS (temp_file, F_OK)) {
@@ -304,7 +309,7 @@ const char *ult_src (const char *name, const char *path,
 	if (flags & SO_LINK) {
 		const char *buffer;
 		char *decomp_base;
-		pipeline *decomp;
+		decompress *decomp;
 		char *include;
 		struct stat st;
 
@@ -327,18 +332,19 @@ const char *ult_src (const char *name, const char *path,
 		 * decompress_open doesn't keep its own copy.
 		 */
 		decomp_base = xstrdup (base);
-		decomp = decompress_open (decomp_base);
+		decomp = decompress_open (decomp_base,
+					  DECOMPRESS_ALLOW_INPROCESS);
 		if (!decomp) {
 			if (quiet < 2)
 				error (0, errno, _("can't open %s"), base);
 			free (decomp_base);
 			return NULL;
 		}
-		pipeline_start (decomp);
+		decompress_start (decomp);
 
 		/* make sure that we skip over any comments */
 		do {
-			buffer = pipeline_readline (decomp);
+			buffer = decompress_readline (decomp);
 		} while (buffer && STRNEQ (buffer, ".\\\"", 3));
 
 		include = test_for_include (buffer);
@@ -361,14 +367,14 @@ const char *ult_src (const char *name, const char *path,
 			free (new_name);
 			recurse--;
 
-			pipeline_wait (decomp);
-			pipeline_free (decomp);
+			decompress_wait (decomp);
+			decompress_free (decomp);
 			free (decomp_base);
 			return ult;
 		}
 
-		pipeline_wait (decomp);
-		pipeline_free (decomp);
+		decompress_wait (decomp);
+		decompress_free (decomp);
 		free (decomp_base);
 	}
 
